"""Admin classes for the ``hero_slider`` app."""
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from hvad.admin import TranslatableAdmin

from . import models


class SliderItemCategoryAdmin(TranslatableAdmin):
    """Admin for the ``SliderItemCategory`` model."""
    list_display = ['slug', 'get_name', 'all_translations']

    def get_name(self, obj):
        return obj.name
    get_name.short_description = _('Name')


class SliderItemAdmin(TranslatableAdmin):
    """Admin for the ``SliderItem`` model."""
    list_display = ['get_title', 'position', 'all_translations',
                    'get_is_published']
    list_editable = ['position', ]

    def get_title(self, obj):
        return obj.title
    get_title.short_description = _('Title')

    def get_is_published(self, obj):
        return obj.is_published
    get_is_published.short_description = _('Is published')
    get_is_published.boolean = True


admin.site.register(models.SliderItem, SliderItemAdmin)
admin.site.register(models.SliderItemCategory, SliderItemCategoryAdmin)



"""Tests for the templatetags of the ``hero_slider`` app."""
from django.template import RequestContext
from django.test import TestCase, RequestFactory

from mixer.backend.django import mixer

from ..templatetags.hero_slider_tags import (
    get_slider_items,
    render_hero_slider,
)


class GetSliderItemsTestCase(TestCase):
    """Tests for the ``get_slider_items`` templatetag."""
    longMessage = True

    def test_tag(self):
        mixer.blend('hero_slider.SliderItemTranslation', language_code='en',
                    is_published=True)
        mixer.blend('hero_slider.SliderItemTranslation', language_code='en',
                    is_published=True)
        req = RequestFactory().get('/')
        req.LANGUAGE_CODE = 'en'
        context = RequestContext(req)
        result = get_slider_items(context)
        self.assertEqual(result.count(), 2, msg=(
            'Should return the published slider items.'))
        result = get_slider_items(context, 1)
        self.assertEqual(result.count(), 1, msg=(
            'Should return the published slider items, limited to 1.'))


class RenderHeroSliderTestCase(TestCase):
    """Tests for the ``render_hero_slider`` templatetag."""
    longMessage = True

    def setUp(self):
        self.de_item = mixer.blend('hero_slider.SliderItem')
        new_item = self.de_item.translate('de')
        new_item.is_published = True
        new_item.save()
        new_item = self.de_item.translate('en')
        new_item.is_published = False
        new_item.save()

        self.en_item = mixer.blend('hero_slider.SliderItem')
        new_item = self.en_item.translate('en')
        new_item.is_published = True
        new_item.save()
        new_item = self.en_item.translate('de')
        new_item.is_published = False
        new_item.save()

    def test_tag(self):
        req = RequestFactory().get('/')
        req.LANGUAGE_CODE = 'en'
        context = RequestContext(req)
        result = render_hero_slider(context)

        self.assertEqual(len(result['slider_items']), 1, msg=(
            'When set to English, it should return one item.'))
        self.assertEqual(
            result['slider_items'][0], self.en_item, msg=(
                'When set to English, it should return the english item.'))

        req.LANGUAGE_CODE = 'de'
        context = RequestContext(req)
        result = render_hero_slider(context)
        self.assertEqual(len(result['slider_items']), 1, msg=(
            'When set to German, it should return one item.'))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-18 01:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_libs.models_mixins
import filer.fields.file


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('filer', '0002_auto_20150606_2003'),
    ]

    operations = [
        migrations.CreateModel(
            name='SliderItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.PositiveIntegerField(blank=True, null=True, verbose_name='Position')),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('external_url', models.CharField(blank=True, max_length=512, verbose_name='External URL')),
                ('text_position_top', models.IntegerField(blank=True, null=True, verbose_name='Text position top')),
                ('text_position_bottom', models.IntegerField(blank=True, null=True, verbose_name='Text position bottom')),
                ('text_position_left', models.IntegerField(blank=True, null=True, verbose_name='Text position left')),
                ('text_position_right', models.IntegerField(blank=True, null=True, verbose_name='Text position right')),
            ],
            options={
                'abstract': False,
            },
            bases=(django_libs.models_mixins.TranslationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SliderItemCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=128, verbose_name='Slug')),
            ],
            options={
                'abstract': False,
            },
            bases=(django_libs.models_mixins.TranslationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SliderItemCategoryTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='hero_slider.SliderItemCategory')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'hero_slider_slideritemcategory_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='SliderItemTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=256, verbose_name='Title')),
                ('description', models.CharField(blank=True, max_length=512, verbose_name='Description')),
                ('link_text', models.CharField(blank=True, max_length=512, verbose_name='Link text')),
                ('is_published', models.BooleanField(default=False, verbose_name='Is published')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='hero_slider.SliderItem')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'hero_slider_slideritem_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.AddField(
            model_name='slideritem',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='hero_slider.SliderItemCategory', verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='slideritem',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='slideritem',
            name='image',
            field=filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, to='filer.File', verbose_name='Image'),
        ),
        migrations.AlterUniqueTogether(
            name='slideritemtranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='slideritemcategorytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
    ]



"""Views for the ``hero_slider`` app."""
import json

from django.contrib.contenttypes.models import ContentType
from django.views.generic import View
from django.http import Http404, HttpResponse


class GetCTypeDetails(View):
    """Returns the app name for a given content type PK as a JSON response."""
    def get(self, request, *args, **kwargs):
        try:
            ctype = ContentType.objects.get(pk=request.GET.get('pk'))
        except ContentType.DoesNotExist:
            raise Http404
        context = {
            'app_label': ctype.app_label,
            'model': ctype.model,
        }
        response_kwargs = {}
        response_kwargs['content_type'] = 'application/json'
        return HttpResponse(json.dumps(context), **response_kwargs)



"""Tests for the models of the ``hero_slider`` app."""
from django.test import TestCase

from mixer.backend.django import mixer


class SliderItemCategoryTestCase(TestCase):
    """Tests for the ``SliderItemCategory`` model."""
    longMessage = True

    def test_model(self):
        instance = mixer.blend('hero_slider.SliderItemCategory')
        self.assertTrue(instance.pk, msg=(
            'Should be able to instantiate and save the model'))


class SliderItemTestCase(TestCase):
    """Tests for the ``SliderItem`` model."""
    longMessage = True

    def setUp(self):
        self.instance = mixer.blend('hero_slider.SliderItem')

    def test_model(self):
        self.assertTrue(self.instance.pk, msg=(
            'Should be able to instantiate and save the model.'))

    def test_get_item_url(self):
        self.assertEqual(
            self.instance.get_item_url(), self.instance.external_url, msg=(
                'Should return the etxernal URL.'))



"""Tests for the views of the ``hero_slider`` app."""
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin

from ..views import GetCTypeDetails


class GetCTypeDetailsTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``GetCTypeDetails`` view class."""
    view_class = GetCTypeDetails

    def test_view(self):
        resp = self.is_callable(data={'pk': 1, })
        ctype = ContentType.objects.get(pk=1)
        self.assertIn(ctype.app_label, resp.content.decode('utf-8'), msg=(
            'Should return a JSON string containing app_label and model'))

    def test_bad_pk(self):
        self.is_not_callable(data={'pk': 999, })



"""Dummy models for the tests of the ``hero_slider`` app."""
from django.db import models


class DummyModel(models.Model):
    """DummyModel for tests of the ``hero_slider`` app."""
    name = models.CharField(max_length=256)



"""URLs for the ``hero_slider`` app."""
from django.conf.urls import url

from .views import GetCTypeDetails


urlpatterns = [
    url(r'ctype/$', GetCTypeDetails.as_view(), name='get_ctype_details'),
]



"""Templatetags for the ``hero_slider`` app."""
from django import template

from ..models import SliderItem


register = template.Library()


@register.assignment_tag(takes_context=True)
def get_slider_items(context, amount=None):
    """Returns the published slider items."""
    req = context.get('request')
    qs = SliderItem.objects.published(req).order_by('position')
    if amount:
        qs = qs[:amount]
    return qs


@register.inclusion_tag('hero_slider/carousel.html', takes_context=True)
def render_hero_slider(context):
    """
    Renders the hero slider.

    """
    req = context.get('request')
    qs = SliderItem.objects.published(req).order_by('position')
    return {
        'slider_items': qs,
    }



"""Models for the ``hero_slider`` app."""
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from django_libs.models_mixins import (
    HvadPublishedManager, TranslationModelMixin)
from filer.fields.file import FilerFileField
from hvad.models import TranslatedFields, TranslatableModel


class SliderItemCategory(TranslationModelMixin, TranslatableModel):
    """
    Each slider item can belong to a category.

    :slug: The slug of this category.

    translated:
    :name: The name of this category

    """
    slug = models.SlugField(
        max_length=128,
        verbose_name=_('Slug'),
    )
    translations = TranslatedFields(
        name=models.CharField(
            max_length=128,
            verbose_name=_('Name'),
        )
    )


class SliderItem(TranslationModelMixin, TranslatableModel):
    """
    Resembles an item that should be shown on the front page in a slider.

    For translateable fields see the ``SliderItemTitle`` model.

    :category: The category of this item (optional)
    :image: A filer file. This will be the image shown in the slider.
    :position: Can be set in order to control the ordering of the slider items.
    :content_type: The contenttype of the object this slider item links to.
    :object_id: The PK of the object this slider item links to.
    :external_url: URL to link to items without the need of a gfk.
    :text_position_top: Text position.
    :text_position_bottom: Text position.
    :text_position_left: Text position.
    :text_position_right: Text position.

    translated:
    :title: The title of this slider item.
    :description: A short description of this slider item.
    :link_text: If you want to render a call to action button on the slide,
      you can define the link text here.
    :is_published: If True, this will not show up.

    """
    category = models.ForeignKey(
        SliderItemCategory,
        verbose_name=_('Category'),
        null=True, blank=True,
    )

    image = FilerFileField(
        verbose_name=_('Image'),
    )

    position = models.PositiveIntegerField(
        verbose_name=_('Position'),
        null=True, blank=True,
    )

    # Generic foreign key
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    external_url = models.CharField(
        verbose_name=_('External URL'),
        max_length=512,
        blank=True,
    )

    text_position_top = models.IntegerField(
        verbose_name=_('Text position top'),
        blank=True, null=True,
    )

    text_position_bottom = models.IntegerField(
        verbose_name=_('Text position bottom'),
        blank=True, null=True,
    )

    text_position_left = models.IntegerField(
        verbose_name=_('Text position left'),
        blank=True, null=True,
    )

    text_position_right = models.IntegerField(
        verbose_name=_('Text position right'),
        blank=True, null=True,
    )

    translations = TranslatedFields(
        title=models.CharField(
            max_length=256,
            verbose_name=_('Title'),
            blank=True,
        ),
        description=models.CharField(
            max_length=512,
            verbose_name=_('Description'),
            blank=True,
        ),
        link_text=models.CharField(
            max_length=512,
            verbose_name=_('Link text'),
            blank=True,
        ),
        is_published=models.BooleanField(
            verbose_name=_('Is published'),
            default=False,
        )
    )

    objects = HvadPublishedManager()

    def get_item_url(self):
        """Returns the url of the connected item."""
        try:
            return self.content_object.get_absolute_url()
        except AttributeError:
            return self.external_url
